local addon, ns = ...

local GetTime = Inspect.Time.Frame
local tinsert, tremove, format = table.insert, table.remove, string.format
local config
local data = {}
local texts = {}
local ctx = UI.CreateContext("RotLatency")
local frame = UI.CreateFrame("Frame", "RotLatency", ctx)
local text = UI.CreateFrame("Text", "Text", frame)
local FormatDuration = ns.FormatDuration

text:SetPoint("TOPLEFT", frame, "TOPLEFT")
text:SetVisible(true)
text:SetText("--RotLatency--")
text:ResizeToText()

frame:SetBackgroundColor(0, 0, 0, 1)
frame:SetVisible(true)

local view = UI.CreateFrame("SimpleScrollView", "Scroll View", ctx)
view:SetContent(frame)
view:SetHeight(100)
view:SetBackgroundColor(0, 0, 0, 1)
view:SetScrollInterval(5)
view:SetShowScrollbar(true)

local height = text:GetHeight()
local width = text:GetWidth()

text:SetMouseMasking("full")

local repositionNow = false
local moveTbl = {function()
	local mouse = Inspect.Mouse()
	view:SetPoint("TOPLEFT", UIParent, "TOPLEFT", mouse.x - 5, mouse.y - 5)
end, "RotLatency", "Move reposition frame"}

text.Event.LeftDown = function()
	if not repositionNow then
		table.insert(Event.System.Update.Begin, moveTbl)
		repositionNow = true
	end
end

text.Event.LeftUp = function()
	if repositionNow then
		for i = #Event.System.Update.Begin, 1, -1 do
			local v = Event.System.Update.Begin[i]
			if v == moveTbl then			
				table.remove(Event.System.Update.Begin, i)
			end
		end
		repositionNow = false
		local mouse = Inspect.Mouse()
		config.x, config.y = mouse.x, mouse.y	
	end
end

table.insert(Event.Addon.SavedVariables.Load.End, {function () 
	config = RotLatency_SavedVariables or {}

	view:SetPoint("TOPLEFT", UIParent, "TOPLEFT", config.x or 40, config.y or 40)
end, "RotLatency", "Load variables"})

table.insert(Event.Addon.SavedVariables.Save.Begin, {function ()

	RotLatency_SavedVariables = config

end, "RotLatency", "Save variables"})

local record = {}
local first = true
table.insert(Event.Ability.Add, {function(abilities)
	--if not first then return end
	first = false
	local count = 0
	for k,v in pairs(abilities or {}) do
		local details = Inspect.Ability.Detail(k)
		if details and details.cooldown then
			count = count + 1
			local text = UI.CreateFrame("Text", "Text"..count, frame)
			text:SetText(details.name)
			text:ResizeToText()
			if text:GetWidth() > width then
				width = text:GetWidth()
			end
			data[k] ={id=k, 
				name=details.name, 
				cooldown=details.cooldown or 1.5, i = count}
			text:SetText(details.name)
			text:ResizeToText()
			text:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, count * height)
			tinsert(texts, text)
		end
	end 
end, "RotLatency", "Available"})

--frame:SetWidth(width)
--frame:SetHeight((count + 1) * height )

table.insert(Event.Ability.Cooldown.Begin, {function(cooldowns)
	local time = GetTime()
	
	for id, v in pairs(cooldowns) do
		local ability = Inspect.Ability.Detail(id)
		if ability and data[id] then
			local remaining = ability.currentCooldownRemaining or 0
			tinsert(data[id], {start=time, remaining=remaining})
		end
	end
end, "RotLatency", "refresh"})


local toDraw = {}
local lastUpdate = GetTime()
table.insert(Event.Ability.Cooldown.End, {function(cooldowns)
	local elapsed = GetTime() - lastUpdate
	lastUpdate = GetTime()
	local time = lastUpdate
	local width = frame:GetWidth()
	local height = text:GetHeight()
	local count = 0
	for id, v in pairs(cooldowns) do
		if data[id] then
			local entry1 = data[id][#data[id]] or {} -- trash
			local entry2 = #data[id] > 1 and data[id][#data[id]-1]
			entry1.finish = GetTime() 
			if entry1 and entry2 and data[id].cooldown then
				local vv = data[id]
				vv.elapsed = (vv.elapsed or 0) + elapsed
				local latency = entry1.start - (entry2.finish or entry1.start)
				if latency > 0.5 then
					vv.elapsed = 0
					vv.lastUpdate = GetTime()
				end

				local perc = (vv.total or 1) / #vv
				vv.latency = latency
				vv.perc = perc
				
			end
		end
	end
	count = 0



end, "RotLatency", "Cooldown.End"})

local lastUpdate = GetTime()
local elapsed = 0
function draw()
	local time = GetTime()
	local delta = GetTime() - lastUpdate
	if delta < .5 then

		elapsed = elapsed + delta
		return
	end
	lastUpdate = time
	for k, v in pairs(texts) do
		v:SetText("")
	end
	local count = 1
	for i, v in pairs(data) do
		local text = texts[count]
		if v.latency then
			v.elapsed = v.elapsed + elapsed
			elapsed = 0
			v.dur = FormatDuration(v.elapsed, "f")
			v.txt = format("(%s) %s: (%.2f sec) - %.2f avg", v.dur, v.name, v.latency, v.perc)
			text:SetText(v.txt)
		else
			text:SetText(v.name)
		end
		
		text:ResizeToText()
		local w = text:GetWidth()
		local h = text:GetHeight()
		if w > width then 
			width = w 
		end
		if h > height then
			height = w
		end
		count = count + 1

	end
		
	view:SetWidth(width + 20)
	frame:SetHeight(height * (count))
end


table.insert(Event.System.Update.Begin, {draw, "RotLatency", "draw"})

table.insert(Command.Slash.Register("rotlatency"), {function (commands)	
	if commands == "reset" then
		config.x, config.y = 40, 40
		view:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 40, 40)
	else
		print("Commands are:")
		print("reset: Resets the window's default position.")
	end
end, "RotLatency", "Slash command"})
